#ifndef _XMA_PARSE_H
#define _XMA_PARSE_H

#include <iostream>

#include "Bit_stream.h"

namespace Parse_XMA {
    using namespace std;

    class Parse_error {
    public:
        virtual void print_self(ostream& os) const = 0;
        friend ostream& operator << (ostream& os, const Parse_error& pe) {
            os << "Parse error: ";
            pe.print_self(os);
            return os;
        }
    };
    class Bad_frame_sync : public Parse_error {
        const unsigned int sync_val;
    public:
        Bad_frame_sync(unsigned int v) : sync_val(v) {}
        void print_self(ostream& os) const {
            os << "unexpected \"frame sync\" " << hex << sync_val<< dec << endl;
        }
    };

    class Early_packet_end : public Parse_error {
    public:
        void print_self(ostream& os) const {
            os << "packet end indicated before end of packet" << endl;
        }
    };

    class Missing_packet_end : public Parse_error {
    public:
        void print_self(ostream& os) const {
            os << "packet end not seen before end of packet" << endl;
        }
    };

    enum {
        packet_size_bytes = 0x800,
        packet_header_size_bytes = 4,
        frame_header_size_bits = 15,
        frame_sync_size_bits = 15,
        frame_trailer_size_bits = 1,
        samples_per_frame = 512
    };

    struct packet_header {
        Bit_uint<6> frame_count;
        Bit_uint<15> skip_bits;
        Bit_uint<3> metadata;
        Bit_uint<8> packet_skip;

        friend Bit_stream& operator>> (Bit_stream& bs, packet_header &ph) {
            return bs >> ph.frame_count >> ph.skip_bits >>
                ph.metadata >> ph.packet_skip;
        }
    };

    /// return sample count
    unsigned int parse_block(istream& is, long offset, long block_size);
}

#endif // _XMA_PARSE
